// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package crystalStructures;
import java.util.ArrayList;

import model.Atom;
import model.AtomData;
import model.ImportStates;

public class BCCExperimentalStructure extends BCCStructure {
	
	@Override
	protected CrystalStructure deriveNewInstance() {
		return new BCCExperimentalStructure();
	}
	
	@Override
	protected String getIDName() {
		return "BCC_Experimental";
	}
	
	@Override
	public boolean isRBVToBeCalculated(Atom a) {
		int type = a.getType();		
		if (type != 6) {			
			return true;
		}
		return false;		
	}
	
	
	@Override
	public ArrayList<Atom> getDislocationDefectAtoms(AtomData data){
		ArrayList<Atom> defectAtoms = new ArrayList<Atom>();
		
		float minRBVLength = latticeConstant*0.866025f*0.15f; //latticeConstant*0.866025 -> perfect burgers vector 1/2[111]
		minRBVLength *= minRBVLength;
		float maxRBVLength = latticeConstant*0.866025f*2.5f;
		maxRBVLength *= maxRBVLength;
		
		for (Atom a : data.getAtoms()) {
			if (a.getType() != 6 && (!ImportStates.POLY_MATERIAL.isActive() || a.getGrain() != Atom.IGNORED_GRAIN)){
				if (data.isRbvAvailable()){
					if (a.getRBV()!=null && a.getRBV().getSquaredRBVLengh()>minRBVLength && a.getRBV().getSquaredRBVLengh()<maxRBVLength)
						defectAtoms.add(a);
				}
				else defectAtoms.add(a);
			}
		}
		return defectAtoms;
	}
}
