// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package model;

import common.ColorTable.ColorBarScheme;

public class CustomColumnInfo {
	private float upperlimit, lowerLimit;
	private String name, id, unit;
	private int column;
	/**
	 * Some columns have a special meaning as the martensite variants of lattice rotation angles
	 * They are stored in the same data array as user imported values, but are not show explicitly in dialogs 
	 */
	private boolean specialColoumn = false;
	private boolean fixedRange = false;
	
	private ColorBarScheme scheme = null;

	public CustomColumnInfo(String name, String id, String unit, int column) {
		this.unit = unit;
		this.name = name.replace(" ", "");
		this.id = id;
		this.column = column;
	}
	
	public CustomColumnInfo(String name, String id, String unit, int column, boolean special) {
		this(name, id, unit, column);
		this.specialColoumn = special;
	}
	
	
	public CustomColumnInfo(String name, String id, String unit, int column, float lowerLimit, float upperLimit, 
			ColorBarScheme scheme) {
		this(name, id, unit, column);
		this.fixedRange = true;
		this.upperlimit = upperLimit;
		this.lowerLimit = lowerLimit;
		this.scheme = scheme;
	}
	
	public float getLowerLimit() {
		return lowerLimit;
	}
	
	public float getUpperLimit() {
		return upperlimit;
	}
	
	public void setUpperLimit(float value){
		if (!fixedRange)
			upperlimit = value;
	}
	
	public void setLowerLimit(float value){
		if (!fixedRange)
			lowerLimit = value;
	}
	
	public String getName() {
		return name;
	}

	public String getId() {
		return id;
	}

	public String getUnit() {
		return unit;
	}
	
	public int getColumn() {
		return column;
	}

	public boolean isSpecialColoumn() {
		return specialColoumn;
	}
	
	public boolean isFixedRange() {
		return fixedRange;
	}
	
	public ColorBarScheme getScheme() {
		return scheme;
	}
	
	public void findRange(boolean global){
		if (fixedRange) return;
		float min = Float.POSITIVE_INFINITY;
		float max = Float.NEGATIVE_INFINITY;
		
		AtomData atomData = Configuration.getCurrentAtomData();
		if (global){
			//rewind to beginning
			while (atomData.getPrevious()!=null)
				atomData = atomData.getPrevious();
			
			while (atomData!=null){
				for (int i=0; i < atomData.getAtoms().size(); i++){
					float v = atomData.getAtoms().get(i).getCustom(column);
					if (v < min) min = v;
					if (v > max) max = v;
				}
				atomData = atomData.getNext();
			}
		} else {
			for (int i=0; i < atomData.getAtoms().size(); i++){
				float v = atomData.getAtoms().get(i).getCustom(column);
				if (v < min) min = v;
				if (v > max) max = v;
			}
		}
		lowerLimit = min;
		upperlimit = max;
	}
	
	@Override
	public String toString(){
		return name;
	}
}