// Part of AtomViewer: AtomViewer is a tool to display and analyse
// atomistic simulations
//
// Copyright (C) 2013  ICAMS, Ruhr-Universität Bochum
//
// AtomViewer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AtomViewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with AtomViewer. If not, see <http://www.gnu.org/licenses/> 

package model;

import gui.JMDFileChooser;
import java.io.*;
import java.util.HashMap;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;

import common.DataInputStreamWrapper;

public class ImdFileLoader extends MDFileLoader{
	
	public ImdFileLoader(JMDFileChooser chooser) {
		super(chooser);
	}

	@Override
	protected AtomData readInputData() throws IOException{
		AtomData toReturn = null;
//		PrintWriter pw = new PrintWriter(new File(chooser.getCurrentDirectory(), "mart.txt"));
						
		if (ImportStates.isImportSequence()){
			String filename = chooser.getSelectedFile().getAbsolutePath();
			String[] parts = filename.split("\\.");
			int num = 0;
			String prefix = parts[0];
			
			String type = ""; 
			if (parts[parts.length-1].equals("gz")) type = parts[parts.length-2];
			else type = parts[parts.length-1];
			
			File f;
			if (parts[parts.length-1].equals("gz")){
				num = Integer.parseInt(parts[parts.length-3]);
				for (int i=1; i<parts.length-3; i++) prefix+="."+parts[i];
				f = new File(prefix+String.format(".%05d.%s.gz", num, type));
			} else {
				num = Integer.parseInt(parts[parts.length-2]);
				for (int i=1; i<parts.length-2; i++) prefix+="."+parts[i];
				f = new File(prefix+String.format(".%05d.%s", num, type));
			}
			AtomData previous = null;
			
			int count = 0; 
						
			while (f.exists() && count < ImportStates.getFilesInSequence() && !isCancelled()){
				firePropertyChange("progress", null, f.getName());
				System.out.println("Processing "+f.getName());
				long time = System.currentTimeMillis();
				AtomData ada = new AtomData(previous, this.readFile(f));
				
				previous = ada;
				num++;
				count++;
				System.out.println("Processing time "+(System.currentTimeMillis()-time));
				
//					String file = f.getName();
//					file = "compact_"+file;
//					File copy = new File(f.getParent(), file);
//					ada.printToFile(copy, true, true);
//					int timestep = (int)(((float[])ada.getFileMetaData("timestep"))[0]);
//					float martVol = 0f;
//					float dis = 0f;
//					for (model.polygrain.Grain g : ada.getGrains()){
//						martVol += g.getMesh().getVolume();
//					}
//					for (model.skeletonizer.Dislocation d : ada.getSkeletonizer().getDislocations()){
//						dis += d.getLength();
//					}
//					pw.printf("%d\t%.4f\t%.4f\n", timestep, martVol, dis);
//					pw.flush();
				
				if (parts[parts.length-1].equals("gz")){
					f = new File(prefix+String.format(".%05d.%s.gz", num, type));
				} else {
					f = new File(prefix+String.format(".%05d.%s", num, type));
				}
			}
			toReturn =  previous;
//				pw.close();
		} else {
			AtomData previous = null;
			for (File f : chooser.getSelectedFiles()){
				firePropertyChange("progress", null, f.getName());
				toReturn = new AtomData(previous, this.readFile(f));
				previous = toReturn;
			}
		}
		
		return toReturn;
	}
	
	
	/**
	 * Read IMD-files, both ASCII and binary versions are supported
	 * @param f File to read
	 * @throws IOException
	 */
	public ImportDataContainer readFile(File f) throws IOException{
		ImportDataContainer idc = new ImportDataContainer();
		idc.name = f.getName();
		LineNumberReader lnr = null;
		if (f.getName().endsWith(".gz")){
			//Directly read gzip-compressed files
			lnr = new LineNumberReader(new InputStreamReader(new GZIPInputStream(new FileInputStream(f))));
		} else 
			lnr = new LineNumberReader(new FileReader(f));
		
		int atomTypeColumn = -1; int elementColumn = -1;
		int xColumn = -1;
		int rbvX_Column = -1;
		int lsX_Column = -1;
		int rbv_data = -1;
		int grainColumn = -1;
		int eulerXColumn = -1;
		int numberColumn = -1;
		
		int[] customsColumns = new int[Configuration.getSizeCustomColumns()];
		for (int i = 0; i<Configuration.getSizeCustomColumns(); i++){
			customsColumns[i] = -1;
		}
		
		int numColumns = 0; 
		String format = "A";
		Pattern p = Pattern.compile("\\s+");
		
		boolean importPerfect = false;
		if (!ImportStates.DISPOSE_DEFAULT.isActive() || ImportStates.BURGERS_VECTORS.isActive() ||
				ImportStates.POLY_MATERIAL.isActive() || ImportStates.DISLOCATION_DENSITY.isActive()) 
			importPerfect = true;
		
		String defectTypeID = Configuration.getCrystalStructure().getAtomTypeKeyword();
		
		String s = lnr.readLine();
		while (s != null && !s.startsWith("#E")) {
			if (s.startsWith("#F")){
				String[] parts = p.split(s);
				format = parts[1];
			}
			if (s.startsWith("#C")) {
				String[] parts = p.split(s);
				numColumns = parts.length - 1;
				for (int i = 0; i < parts.length; i++) {
					if (parts[i].equals("number")) numberColumn = i - 1;
					if (!ImportStates.OVERRIDE.isActive() && parts[i].equals(defectTypeID)) atomTypeColumn = i - 1;
					if (parts[i].equals("type")) elementColumn = i - 1;
					if (parts[i].equals("x")) xColumn = i - 1;
					if (parts[i].equals("rbv_data")) rbv_data = i - 1;
					if (!ImportStates.OVERRIDE.isActive() && parts[i].equals("rbv_x")) rbvX_Column = i - 1;
					if (!ImportStates.OVERRIDE.isActive() && parts[i].equals("ls_x")) lsX_Column = i - 1;
					if (!ImportStates.OVERRIDE.isActive() && parts[i].equals("grain")) grainColumn = i - 1;
					if (!ImportStates.OVERRIDE.isActive() && parts[i].equals("eulerX")) eulerXColumn = i - 1;
					
					for (int j = 0; j<Configuration.getSizeCustomColumns(); j++){
						if (parts[i].equals(Configuration.getCustomColumnInfo(j).getId()))
							customsColumns[j] = i - 1;
					}
				}
			} else if (s.startsWith("#X")){
				String[] parts = p.split(s);
				idc.boxSize[0] = Float.parseFloat(parts[1]);
			} else if (s.startsWith("#Y")){
				String[] parts = p.split(s);
				idc.boxSize[1] = Float.parseFloat(parts[2]);
			} else if (s.startsWith("#Z")){
				String[] parts = p.split(s);
				idc.boxSize[2] = Float.parseFloat(parts[3]);
			} else if (s.startsWith("##META")){
				s = lnr.readLine();
				if (idc.fileMetaData == null)
					idc.fileMetaData = new HashMap<String, Object>();
				while (!s.startsWith("##METAEND")) {
					//First check if there are custom options
					if (s.startsWith("##") &&
							!Configuration.getCrystalStructure().processMetadataLine(s, idc.fileMetaData, lnr, idc)){
						//Try to store the line as a array of floats, e.g. timesteps, indenter...
						try{
							s = s.substring(2);
							String[] parts = p.split(s);
							float[] info = new float[parts.length-1];
							for (int i=1; i<parts.length; i++){
								info[i-1] = Float.parseFloat(parts[i]);
							}
							idc.fileMetaData.put(parts[0].toLowerCase(), info);
						} catch (Exception e) {}
					}
					s = lnr.readLine();
				}
			}
			s = lnr.readLine();
		}
		
		if (xColumn ==-1) 
			throw new IllegalArgumentException("Broken header, no coordinates x y z");
		
		if (idc.boxSize[0] <= 0f || idc.boxSize[1] <= 0f || idc.boxSize[2] <= 0f){
			throw new IllegalArgumentException("Broken header, box sizes must be larger than 0");
		}
		
		//Short circuit if only stresses are to be imported
		if (ImportStates.KILL_ALL_ATOMS.isActive() && !ImportStates.SKELETONIZE.isActive() && 
				!ImportStates.POLY_MATERIAL.isActive() && !ImportStates.DISLOCATION_DENSITY.isActive())
			return idc;
		
		boolean readRBV = false;
		if ( !ImportStates.OVERRIDE.isActive() && ((rbvX_Column!=-1 && lsX_Column!=-1 ) || rbv_data!=-1) &&
				ImportStates.BURGERS_VECTORS.isActive()){
			readRBV = true;
			idc.rbvAvailable = true;
		}
		
		if (atomTypeColumn != -1) idc.atomTypesAvailable = true;
		else importPerfect = true;
		if (grainColumn != -1) idc.grainsImported = true;
		if (eulerXColumn != -1 && ImportStates.LATTICE_ROTATION.isActive()) idc.latticeRotationImported = true;
		
		try{
			if (format.equals("A")){	//Ascii-Format
				int number = 0;
				s = lnr.readLine();
				float[] pos = new float[3];
				int[] typeNumEle = new int[3];
				typeNumEle[0] = 3;
				while (s!=null){
					s = s.trim();
					if (s.isEmpty()){ /*Skipping empty lines if someone inserted them in the file */
						s = lnr.readLine();
						continue;
					}
					String[] parts = p.split(s);
					
					int grain = 0;
					
					if (atomTypeColumn!=-1) typeNumEle[0] = Integer.parseInt(parts[atomTypeColumn]);
					if (grainColumn!=-1) {
						grain = Integer.parseInt(parts[grainColumn]);
					}
					if (elementColumn!=-1) {
						typeNumEle[2] = Integer.parseInt(parts[elementColumn]);
						if (typeNumEle[2] + 1 > idc.maxNumElements) idc.maxNumElements = typeNumEle[2] + 1;
					}
					
					if ( importPerfect || typeNumEle[0]!=Configuration.getCrystalStructure().getDefaultType()) {
						if (numberColumn == -1) typeNumEle[1] = number++;
						else typeNumEle[1] = Integer.parseInt(parts[numberColumn]);
						
						pos[0] = Float.parseFloat(parts[xColumn]);
						pos[1] = Float.parseFloat(parts[xColumn+1]);
						pos[2] = Float.parseFloat(parts[xColumn+2]);
						
						//Put atoms back into the simulation box, they might be slightly outside
						idc.backInBox(pos);

						Atom a = new Atom(pos, typeNumEle);
						
						if (grainColumn!=-1 && ImportStates.POLY_MATERIAL.isActive()) a.setGrain(grain);
						idc.atoms.add(a);
						
						//Custom columns
						for (int i = 0; i<customsColumns.length; i++){
							if (customsColumns[i]!=-1)
								a.setCustom(Float.parseFloat(parts[customsColumns[i]]), i);
						}
						
						if (readRBV){
							float[] rbv = new float[3];
							float[] lineDirection = new float[3];

							if (rbv_data == -1){
								rbv[0] = Float.parseFloat(parts[rbvX_Column]);
								rbv[1] = Float.parseFloat(parts[rbvX_Column+1]);
								rbv[2] = Float.parseFloat(parts[rbvX_Column+2]);
									
								lineDirection[0] = Float.parseFloat(parts[lsX_Column]);
								lineDirection[1] = Float.parseFloat(parts[lsX_Column+1]);
								lineDirection[2] = Float.parseFloat(parts[lsX_Column+2]);
							} else {
								int data = Integer.parseInt(parts[rbv_data]);
								if (data == 1){
									lineDirection[0] = Float.parseFloat(parts[rbv_data+1]);
									lineDirection[1] = Float.parseFloat(parts[rbv_data+2]);
									lineDirection[2] = Float.parseFloat(parts[rbv_data+3]);
									
									rbv[0] = Float.parseFloat(parts[rbv_data+4]);
									rbv[1] = Float.parseFloat(parts[rbv_data+5]);
									rbv[2] = Float.parseFloat(parts[rbv_data+6]);
								}
							}

							if (rbv[0]!=0f && rbv[1]!=0f && rbv[2]!=0f)
								a.setRBV(rbv, lineDirection);
						}
					}
					s = lnr.readLine();
				}
			}
			//binary formats
			else if (format.equals("l") || format.equals("b") || format.equals("L") || format.equals("B")){
				if (numberColumn == -1) throw new IllegalArgumentException("binary files must contain number");
				FileInputStream fis = new FileInputStream(f);
				BufferedInputStream bis;
				long filesize = 0l;
				int defaultType = Configuration.getCrystalStructure().getDefaultType();
				
				if (f.getName().endsWith(".gz")){
					//Setup streams for compressed files
					//read streamSize from the last 4 bytes in the file
					RandomAccessFile raf = new RandomAccessFile(f, "r");
					long l = raf.length();
					raf.seek(l-4);
					
					byte[] w = new byte[4];
					raf.readFully(w, 0, 4);
					filesize = (w[3]) << 24 | (w[2]&0xff) << 16 | (w[1]&0xff) << 8 | (w[0]&0xff);
					raf.close();
					bis = new BufferedInputStream(new GZIPInputStream(fis), 16384);
				} else {
					bis = new BufferedInputStream(fis, 16384);
					filesize = fis.available();
				}
				
				boolean littleEndian = format.equals("l") || format.equals("L");
				boolean doublePrecision = format.equals("L") || format.equals("B");
				
				DataInputStreamWrapper dis = 
					DataInputStreamWrapper.getDataInputStreamWrapper(bis, doublePrecision, littleEndian);
				try {
					//Seek the end of the header
					boolean terminationSymbolFound = false;
					byte b1, b2 = 0;
					do {
						b1 = b2;
						b2 = dis.readByte();
						if (b1 == 0x23 && b2 == 0x45) terminationSymbolFound = true; 
					} while (!terminationSymbolFound);
					//read end of line - lf for unix or cr/lf for windows
					byte b = dis.readByte();
					if (b == 0x0d) {
						b = dis.readByte();
					}
					
					float[] pos = new float[3];
					float[] rbv = new float[3];
					float[] lineDirection = new float[3];
					int[] typeNumEle = new int[3];
					
					float[] customColumnValues = new float[customsColumns.length];
					typeNumEle[0] = 3;
					int grain = 0;
					
					while (filesize > dis.getBytesRead()){
						for (int i = 0; i < numColumns; i++){
							if (i == 0){
								//number and type are always packed
								typeNumEle[1] = dis.readIntSingle();
								typeNumEle[2] = dis.readIntSingle();

								if (typeNumEle[2]+1 > idc.maxNumElements) idc.maxNumElements = typeNumEle[2] + 1;
								//The data format is ugly, both type and number are always packed in 8 bytes
								//no matter of double precision or not
								//so the next entry has to be skipped
								++i;
							} else if (i==xColumn){
								pos[0] = dis.readFloat();
								pos[1] = dis.readFloat();
								pos[2] = dis.readFloat();
								i+=2;
							} else if (i==rbv_data){
								if (dis.readInt() == 1){
									lineDirection[0] = dis.readFloat();
									lineDirection[1] = dis.readFloat();
									lineDirection[2] = dis.readFloat();
									rbv[0] = dis.readFloat();
									rbv[1] = dis.readFloat();
									rbv[2] = dis.readFloat();
								}
							} else if (i==rbvX_Column){
								rbv[0] = dis.readFloat();
								rbv[1] = dis.readFloat();
								rbv[2] = dis.readFloat();
								i+=2;
							} else if (i==lsX_Column){
								lineDirection[0] = dis.readFloat();
								lineDirection[1] = dis.readFloat();
								lineDirection[2] = dis.readFloat();
								i+=2;
							} else if (i==atomTypeColumn){
								typeNumEle[0] = dis.readInt();
							} else if (i==grainColumn){
								grain = dis.readInt();
							} else {
								if (customColumnValues.length!=0){
									boolean read = false;
									for (int j = 0; j<customsColumns.length; j++){
										if (i == customsColumns[j]){
											customColumnValues[j] = dis.readFloat();
											read = true;
											break;
										}
									}
									if (!read) dis.readInt();
								}
								else dis.readInt();
							}
						} 

						if ( importPerfect || typeNumEle[0]!=defaultType) {
							//Put atoms back into the simulation box, they might be slightly outside
							idc.backInBox(pos);
							
							Atom a = new Atom(pos, typeNumEle);
							//Add rbv info is found in file
							if (readRBV && !Float.isNaN(rbv[0])){
								a.setRBV(rbv, lineDirection);
								rbv[0] = Float.NaN;
							}
							if (grainColumn!=-1 && ImportStates.POLY_MATERIAL.isActive()) 
								a.setGrain(grain);	//Assign grain number if found
							idc.atoms.add(a); //Put atom into the list of all atoms

							//Custom columns
							for (int i = 0; i<customsColumns.length; i++){
								if (customsColumns[i]!=-1)
									a.setCustom(customColumnValues[i], i);
							}
						}
					}
					
				} catch (IOException ex) {
					throw ex;
				} finally {
					dis.close();
				}
			}
			else throw new IllegalArgumentException("File format not supported");
			
			//Handling of special case using customColumns, such as reading x,y,z
			for (int j = 0; j<Configuration.getSizeCustomColumns(); j++){
				if (Configuration.getCustomColumnInfo(j).getId().equals("x")){
					for (Atom a : idc.atoms)
						a.setCustom(a.getX(), j);
				}
				if (Configuration.getCustomColumnInfo(j).getId().equals("y")){
					for (Atom a : idc.atoms)
						a.setCustom(a.getY(), j);
				}
				if (Configuration.getCustomColumnInfo(j).getId().equals("z")){
					for (Atom a : idc.atoms)
						a.setCustom(a.getZ(), j);
				}
			}
		}catch (IOException ex){
			throw ex;
		} finally {
			lnr.close();
		}
		return idc;
	}
}