package model.polygrain.kdTree;

import model.Coordinate;
import common.VectorOps;

/**
 * Original code released under the Gnu Lesser General Public License:
 * http://home.wlu.edu/~levys/software/kd/
 */
class KDNode<T extends Coordinate> {
	T value;
	protected KDNode<T> left, right;

	protected  void insert(T val, int lev) {
		KDNode<T> next_node = null;

		float p,q;
		if (lev == 0){
			p = val.getX();
			q = this.value.getX();
		} else if (lev == 1){
			p = val.getY();
			q = this.value.getY();
		} else {
			p = val.getZ();
			q = this.value.getZ();
		}
		
		if (p > q) {
			next_node = this.right;
			if (next_node == null) {
				this.right = new KDNode<T>(val);
				return;
			}
		} else {
			next_node = this.left;
			if (next_node == null) {
				this.left = new KDNode<T>(val);
				return;
			}
		}

		int next_lev = (lev + 1) % 3;
		next_node.insert(val, next_lev);
	}

	protected static <T extends Coordinate> void nnbr(KDNode<T> kd, Coordinate target, HRect hr, int lev, NearestNeighbor<KDNode<T>> nnl) {
		if (kd == null)
			return;

		//level%3 simplified
		if (lev == 3) lev = 0;

		float sqrDistToTarget = VectorOps.sqrDist(kd.value, target);

		// 4. Cut hr into to sub-hyperrectangles left-hr and right-hr.
		// The cut plane is through pivot and perpendicular to the s
		// dimension.
		HRect left_hr = hr;
		HRect right_hr = hr.clone();
		float p, q;
		
		if (lev == 0){
			left_hr.maxX = kd.value.getX();
			right_hr.minX = kd.value.getX();
			q = kd.value.getX();
			p = target.getX();
		} else if (lev == 1){
			left_hr.maxY = kd.value.getY();
			right_hr.minY = kd.value.getY();
			q = kd.value.getY();
			p = target.getY();
		} else {
			left_hr.maxZ = kd.value.getZ();
			right_hr.minZ = kd.value.getZ();
			q = kd.value.getZ();
			p = target.getZ();
		}
		KDNode<T> further_kd;
		HRect further_hr;

		// 6. if target-in-left then
		// 6.1. nearer-kd := left field of kd and nearer-hr := left-hr
		// 6.2. further-kd := right field of kd and further-hr := right-hr
		if (p < q) {
			// 8. Recursively call Nearest Neighbor with paramters
			// (nearer-kd, target, nearer-hr, max-dist-sqd), storing the
			// results in nearest and dist-sqd
			nnbr(kd.left, target, left_hr, lev + 1, nnl);
			further_kd = kd.right;
			further_hr = right_hr;
		} else {
			// 7. if not target-in-left then
			// 7.1. nearer-kd := right field of kd and nearer-hr := right-hr
			// 7.2. further-kd := left field of kd and further-hr := left-hr
			nnbr(kd.right, target, right_hr, lev + 1, nnl);
			further_kd = kd.left;
			further_hr = left_hr;
		}

		// 10. A nearer point could only lie in further-kd if there were some
		// part of further-hr within distance max-dist-sqd of
		// target.
		float[] closest = further_hr.closest(target);
		if (VectorOps.sqrDist(closest, target) < nnl.value) {
			if (sqrDistToTarget < nnl.value)
				nnl.set(kd, sqrDistToTarget);
			
			// 10.2 Recursively call Nearest Neighbor with parameters
			// (further-kd, target, further-hr, max-dist_sqd),
			// storing results in temp-nearest and temp-dist-sqd
			nnbr(further_kd, target, further_hr, lev + 1, nnl);
		}
	}

	protected KDNode(T val) {
		value = val;
		left = null;
		right = null;
	}
}
